#! /usr/bin/perl -w
# File checksum.pl created by Michael McEvoy. 
# Modified by Randal Anderson on Thu Jan 23 2003. 

my $pathName = "";
my $extension = "";
my $line;


# first argument must be the name of the image
my $imageName = shift(@ARGV);
if( !defined($imageName))
{ SayHelp(); }
if( !-e( "$imageName" ))
{
  $extension = ".img";
  if( !-e( "$imageName$extension" ))
  {
    $pathName = "/tmp/fwupdate/";
    $extension = "";
    if( !-e( "$pathName$imageName" ))
    {
      $extension = ".img";
      if( !-e( "$pathName$imageName$extension" ))
      {
        print "Image $imageName not found\n";
        SayHelp();
      }
    }
  }
}

# checksum file of same name + .chksum must be in same directory
my $checkName = $imageName;
$checkName =~ s/\.img//i;
if( !-e( "$pathName$checkName.chksum" ))
{
  print "Checksum file $pathName$checkName.chksum not found\n";
  SayHelp();
}

# should be okay to check it
print "Verifying checksums for $pathName$imageName$extension against $pathName$checkName.chksum\n";

# open and parse stored checksum file
open(CHKSUM, "$pathName$checkName.chksum") ||die "Can't open checksum file:$!\n";
$line = <CHKSUM>;
$line =~ /^(\d+)\s+(\d+)/;
my $chksum = $1;
my $blocks = $2;
close(CHKSUM);
print "Checksum file $pathName$checkName.chksum: checksum=$chksum  blocks=$blocks\n";

# query image file for current checksum for comparison
my $chksum2_str = "sum $pathName$imageName$extension";
open(QUERY, "$chksum2_str |");
$line = <QUERY>;
close(QUERY);

if(!$line)
{ print "Checksum file is empty.\n"; SayHelp(); }

$line =~ /^(\d+)\s+(\d+)/;
my $chksum2 = $1;
my $blocks2 = $2;
print "Image file $pathName$imageName$extension: checksum=$chksum2  blocks=$blocks2\n";

# report results
my $Bad = "";
if($chksum != $chksum2)
{ $Bad .= "*** Checksum Mismatch  "; }
if($blocks != $blocks2)
{ $Bad .= "*** Block Size Mismatch  "; }
if( $Bad ne "" )
{ print "$Bad\n"; exit(1); }

print "The checksums and block sizes match!\n";
exit(0);


#------------------------------------------------------------------------
# SayHelp: Print out help text
#------------------------------------------------------------------------
sub SayHelp
{
  print STDERR "
Usage:  checksum.pl <imageName>
        <imageName> is the name of the image to do a checksum on.
                    an extension of .img is assumed.
                    the checksum file will be <imageName>.chksum
";

exit(1);
}  #-- end SayHelp
